@@ -3,6 +3,7 @@ import logging
 
 from sqlite3 import Error
 
+from moodlerpd.state.category import Category
 from moodlerpd.state.file import File
 from moodlerpd.state.course import Course
 
@@ -12,6 +13,7 @@ class StateService:
     Saves the state and provides utilities to detect changes in the current
     state against the previous.
     """
+    categories_list: [Category] = []
 
     def __init__(self, db_file: str):
         """
@@ -29,6 +31,7 @@ class StateService:
             sql_create_index_table = """ CREATE TABLE IF NOT EXISTS files (
             course_id integer NOT NULL,
             course_fullname integer NOT NULL,
+            course_category_id integer NULL,
             module_id integer NOT NULL,
             section_name text NOT NULL,
             module_name text NOT NULL,
@@ -165,7 +168,7 @@ class StateService:
             conn.close()
 
         except Error as error:
-            raise RuntimeError('Could not create database! Error: %s' % (error))
+            raise RuntimeError('Could not create database! Error: %s' % error)
 
     @staticmethod
     def __files_have_same_type(file1: File, file2: File) -> bool:
@@ -246,7 +249,7 @@ class StateService:
 
         return False
 
-    def get_stored_files(self) -> [Course]:
+    def get_stored_courses(self) -> [Course]:
         # get all stored files (that are not yet deleted)
         conn = sqlite3.connect(self.db_file)
         conn.row_factory = sqlite3.Row
@@ -254,7 +257,7 @@ class StateService:
         stored_courses = []
 
         cursor.execute(
-            """SELECT course_id, course_fullname
+            """SELECT course_id, course_fullname, course_category_id
             FROM files WHERE deleted = 0 AND modified = 0 AND moved = 0
             GROUP BY course_id;"""
         )
@@ -262,7 +265,8 @@ class StateService:
         curse_rows = cursor.fetchall()
 
         for course_row in curse_rows:
-            course = Course(course_row['course_id'], course_row['course_fullname'], [])
+            course = Course(course_row['course_id'], course_row['course_fullname'],
+                            course_row['course_category_id'])
 
             cursor.execute(
                 """SELECT *
@@ -276,6 +280,8 @@ class StateService:
 
             file_rows = cursor.fetchall()
 
+            course.categories = course.build_categories(course.category_id, self.categories_list)
+
             course.files = []
 
             for file_row in file_rows:
@@ -437,7 +443,7 @@ class StateService:
         # later check for new files
 
         # first get all stored files (that are not yet deleted)
-        stored_courses = self.get_stored_files()
+        stored_courses = self.get_stored_courses()
 
         changed_courses = self.__get_modified_files(stored_courses, current_courses)
         # ----------------------------------------------------------
@@ -478,14 +484,15 @@ class StateService:
         cursor = conn.cursor()
 
         cursor.execute(
-            """SELECT course_id, course_fullname
+            """SELECT course_id, course_fullname, course_category_id
             FROM files WHERE notified = 0 GROUP BY course_id;"""
         )
 
         curse_rows = cursor.fetchall()
 
         for course_row in curse_rows:
-            course = Course(course_row['course_id'], course_row['course_fullname'], [])
+            course = Course(course_row['course_id'], course_row['course_fullname'],
+                            course_row['course_category_id'])
 
             cursor.execute(
                 """SELECT *
@@ -495,6 +502,8 @@ class StateService:
 
             file_rows = cursor.fetchall()
 
+            course.categories = course.build_categories(course.category_id, self.categories_list)
+
             course.files = []
 
             for file_row in file_rows:
@@ -544,23 +553,25 @@ class StateService:
         conn.commit()
         conn.close()
 
-    def save_file(self, file: File, course_id: int, course_fullname: str):
+    def save_file(self, file: File, course_id: int, course_fullname: str, course_category_id: int):
         if file.deleted:
-            self.delete_file(file, course_id, course_fullname)
+            self.delete_file(file, course_id, course_fullname, course_category_id)
         elif file.modified:
-            self.modifie_file(file, course_id, course_fullname)
+            self.modifie_file(file, course_id, course_fullname, course_category_id)
         elif file.moved:
-            self.move_file(file, course_id, course_fullname)
+            self.move_file(file, course_id, course_fullname, course_category_id)
         else:
-            self.new_file(file, course_id, course_fullname)
+            self.new_file(file, course_id, course_fullname, course_category_id)
 
-    def new_file(self, file: File, course_id: int, course_fullname: str):
+    def new_file(self, file: File, course_id: int, course_fullname: str, course_category_id: int):
         # saves a file to index
 
         conn = sqlite3.connect(self.db_file)
         cursor = conn.cursor()
 
-        data = {'course_id': course_id, 'course_fullname': course_fullname}
+        data = {'course_id': course_id, 'course_fullname': course_fullname,
+                'course_category_id': course_category_id}
+
         data.update(file.get_map())
 
         data.update({'modified': 0, 'deleted': 0, 'moved': 0, 'notified': 0})
@@ -577,7 +588,9 @@ class StateService:
         for course in courses:
             for file in course.files:
                 if file.deleted:
-                    data = {'course_id': course.id, 'course_fullname': course.fullname}
+                    data = {'course_id': course.id, 'course_fullname': course.fullname,
+                            'course_category_id': course.category_id}
+
                     data.update(file.get_map())
 
                     cursor.execute(
@@ -609,11 +622,13 @@ class StateService:
         conn.commit()
         conn.close()
 
-    def delete_file(self, file: File, course_id: int, course_fullname: str):
+    def delete_file(self, file: File, course_id: int, course_fullname: str, course_category_id: int):
         conn = sqlite3.connect(self.db_file)
         cursor = conn.cursor()
 
-        data = {'course_id': course_id, 'course_fullname': course_fullname}
+        data = {'course_id': course_id, 'course_fullname': course_fullname,
+                'course_category_id': course_category_id}
+
         data.update(file.get_map())
 
         cursor.execute(
@@ -627,11 +642,13 @@ class StateService:
         conn.commit()
         conn.close()
 
-    def move_file(self, file: File, course_id: int, course_fullname: str):
+    def move_file(self, file: File, course_id: int, course_fullname: str, course_category_id: int):
         conn = sqlite3.connect(self.db_file)
         cursor = conn.cursor()
 
-        data_new = {'course_id': course_id, 'course_fullname': course_fullname}
+        data_new = {'course_id': course_id, 'course_fullname': course_fullname,
+                    'course_category_id': course_category_id}
+
         data_new.update(file.get_map())
 
         if file.old_file is not None:
@@ -643,7 +660,9 @@ class StateService:
             )
             cursor.execute(File.INSERT, data_new)
 
-            data_old = {'course_id': course_id, 'course_fullname': course_fullname}
+            data_old = {'course_id': course_id, 'course_fullname': course_fullname,
+                        'course_category_id': course_category_id}
+
             data_old.update(file.old_file.get_map())
 
             cursor.execute(
@@ -663,11 +682,13 @@ class StateService:
         conn.commit()
         conn.close()
 
-    def modifie_file(self, file: File, course_id: int, course_fullname: str):
+    def modifie_file(self, file: File, course_id: int, course_fullname: str, course_category_id: int):
         conn = sqlite3.connect(self.db_file)
         cursor = conn.cursor()
 
-        data_new = {'course_id': course_id, 'course_fullname': course_fullname}
+        data_new = {'course_id': course_id, 'course_fullname': course_fullname,
+                    'course_category_id': course_category_id}
+
         data_new.update(file.get_map())
 
         if file.old_file is not None:
@@ -679,7 +700,9 @@ class StateService:
             )
             cursor.execute(File.INSERT, data_new)
 
-            data_old = {'course_id': course_id, 'course_fullname': course_fullname}
+            data_old = {'course_id': course_id, 'course_fullname': course_fullname,
+                        'course_category_id': course_category_id}
+
             data_old.update(file.old_file.get_map())
 
             cursor.execute(
@@ -700,3 +723,6 @@ class StateService:
 
         conn.commit()
         conn.close()
+
+    def update_categories(self, categories_list: [Category]):
+        self.categories_list = categories_list
